@@ -1,12 +1,11 @@
 //! Structured representations of SPIR-V types.
-use std::collections::BTreeMap;
 use std::fmt;
-use crate::MemberVariableResolution;
+use std::hash::Hash;
 use crate::error::*;
-use crate::sym::{Sym, Seg, Symbol};
-use std::hash::{Hash, Hasher};
+use crate::walk::Walk;
 
-pub use spirv_headers::{Dim, ImageFormat};
+use spirv_headers::Dim;
+pub use spirv_headers::{ImageFormat};
 
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub enum ScalarType {
@@ -140,8 +139,13 @@ impl fmt::Debug for MatrixType {
 
 #[derive(PartialEq, Eq, Hash, Clone, Copy)]
 pub enum ImageUnitFormat {
+    /// The image is used as a storage image and the read/write format is
+    /// explicitly specified.
     Color(ImageFormat),
+    /// The image is used as a sampled image.
     Sampled,
+    /// The image is used as a sampled depth image. Note that you cannot access
+    /// depth-stencil images by read/write.
     Depth,
 }
 impl ImageUnitFormat {
@@ -197,13 +201,19 @@ impl ImageArrangement {
 
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub struct ImageType {
-    pub scalar_ty: ScalarType,
+    /// Scalar type of image access result. In most cases it's `Some`, but the
+    /// SPIR-V specification allows it to be `OpTypeVoid`. I have never
+    /// encounter one tho.
+    pub scalar_ty: Option<ScalarType>,
+    /// Matches `VkImageCreateInfo::format`.
     pub unit_fmt: ImageUnitFormat,
+    /// Image arrangement which encodes multisampling, array-ness and
+    /// dimentionality.
     pub arng: ImageArrangement,
 }
 impl ImageType {
     pub fn new(
-        scalar_ty: ScalarType,
+        scalar_ty: Option<ScalarType>,
         unit_fmt: ImageUnitFormat,
         arng: ImageArrangement,
     ) -> ImageType {
@@ -214,6 +224,14 @@ impl fmt::Debug for ImageType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         use ImageArrangement::*;
         use ImageUnitFormat::*;
+        let scalar_ty = || -> String {
+            let scalar_ty = self.scalar_ty.as_ref();
+            if let Some(scalar_ty) = scalar_ty {
+                format!("{:?}", scalar_ty)
+            } else {
+                "void".to_owned()
+            }
+        };
         match (self.arng, self.unit_fmt) {
             (Image1D, Color(fmt)) => write!(f, "image1D<{:?}>", fmt),
             (Image2D, Color(fmt)) => write!(f, "image2D<{:?}>", fmt),
@@ -227,17 +245,17 @@ impl fmt::Debug for ImageType {
             (Image2DRect, Color(fmt)) => write!(f, "image2DRect<{:?}>", fmt),
             (ImageBuffer, Color(fmt)) => write!(f, "imageBuffer<{:?}>", fmt),
 
-            (Image1D, Sampled) => write!(f, "texture1D<{:?}>", self.scalar_ty),
-            (Image2D, Sampled) => write!(f, "texture2D<{:?}>", self.scalar_ty),
-            (Image2DMS, Sampled) => write!(f, "texture2DMS<{:?}>", self.scalar_ty),
-            (Image3D, Sampled) => write!(f, "texture3D<{:?}>", self.scalar_ty),
-            (CubeMap, Sampled) => write!(f, "textureCube<{:?}>", self.scalar_ty),
-            (Image1DArray, Sampled) => write!(f, "texture1DArray<{:?}>", self.scalar_ty),
-            (Image2DArray, Sampled) => write!(f, "texture2DArray<{:?}>", self.scalar_ty),
-            (Image2DMSArray, Sampled) => write!(f, "texture2DMSArray<{:?}>", self.scalar_ty),
-            (CubeMapArray, Sampled) => write!(f, "textureCubeArray<{:?}>", self.scalar_ty),
-            (Image2DRect, Sampled) => write!(f, "texture2DRect<{:?}>", self.scalar_ty),
-            (ImageBuffer, Sampled) => write!(f, "textureBuffer<{:?}>", self.scalar_ty),
+            (Image1D, Sampled) => write!(f, "texture1D<{}>", scalar_ty()),
+            (Image2D, Sampled) => write!(f, "texture2D<{}>", scalar_ty()),
+            (Image2DMS, Sampled) => write!(f, "texture2DMS<{}>", scalar_ty()),
+            (Image3D, Sampled) => write!(f, "texture3D<{}>", scalar_ty()),
+            (CubeMap, Sampled) => write!(f, "textureCube<{}>", scalar_ty()),
+            (Image1DArray, Sampled) => write!(f, "texture1DArray<{}>", scalar_ty()),
+            (Image2DArray, Sampled) => write!(f, "texture2DArray<{}>", scalar_ty()),
+            (Image2DMSArray, Sampled) => write!(f, "texture2DMSArray<{}>", scalar_ty()),
+            (CubeMapArray, Sampled) => write!(f, "textureCubeArray<{}>", scalar_ty()),
+            (Image2DRect, Sampled) => write!(f, "texture2DRect<{}>", scalar_ty()),
+            (ImageBuffer, Sampled) => write!(f, "textureBuffer<{}>", scalar_ty()),
             _ => Err(fmt::Error::default()),
         }
     }
@@ -246,7 +264,7 @@ impl fmt::Debug for ImageType {
 
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub struct SampledImageType {
-    img_ty: ImageType,
+    pub img_ty: ImageType,
 }
 impl SampledImageType {
     pub fn new(img_ty: ImageType) -> SampledImageType {
@@ -257,26 +275,34 @@ impl fmt::Debug for SampledImageType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         use ImageArrangement::*;
         use ImageUnitFormat::*;
+        let scalar_ty = {
+            let scalar_ty = self.img_ty.scalar_ty.as_ref();
+            if let Some(scalar_ty) = scalar_ty {
+                format!("{:?}", scalar_ty)
+            } else {
+                "void".to_owned()
+            }
+        };
         match (self.img_ty.arng, self.img_ty.unit_fmt) {
-            (Image1D, Sampled) => write!(f, "sampler1D<{:?}>", self.img_ty.scalar_ty),
-            (Image2D, Sampled) => write!(f, "sampler2D<{:?}>", self.img_ty.scalar_ty),
-            (Image2DMS, Sampled) => write!(f, "sampler2DMS<{:?}>", self.img_ty.scalar_ty),
-            (Image3D, Sampled) => write!(f, "sampler3D<{:?}>", self.img_ty.scalar_ty),
-            (CubeMap, Sampled) => write!(f, "samplerCube<{:?}>", self.img_ty.scalar_ty),
-            (Image1DArray, Sampled) => write!(f, "sampler1DArray<{:?}>", self.img_ty.scalar_ty),
-            (Image2DArray, Sampled) => write!(f, "sampler2DArray<{:?}>", self.img_ty.scalar_ty),
-            (Image2DMSArray, Sampled) => write!(f, "sampler2DMSArray<{:?}>", self.img_ty.scalar_ty),
-            (CubeMapArray, Sampled) => write!(f, "samplerCubeArray<{:?}>", self.img_ty.scalar_ty),
-            (Image2DRect, Sampled) => write!(f, "sampler2DRect<{:?}>", self.img_ty.scalar_ty),
-            (ImageBuffer, Sampled) => write!(f, "samplerBuffer<{:?}>", self.img_ty.scalar_ty),
+            (Image1D, Sampled) => write!(f, "sampler1D<{}>", scalar_ty),
+            (Image2D, Sampled) => write!(f, "sampler2D<{}>", scalar_ty),
+            (Image2DMS, Sampled) => write!(f, "sampler2DMS<{}>", scalar_ty),
+            (Image3D, Sampled) => write!(f, "sampler3D<{}>", scalar_ty),
+            (CubeMap, Sampled) => write!(f, "samplerCube<{}>", scalar_ty),
+            (Image1DArray, Sampled) => write!(f, "sampler1DArray<{}>", scalar_ty),
+            (Image2DArray, Sampled) => write!(f, "sampler2DArray<{}>", scalar_ty),
+            (Image2DMSArray, Sampled) => write!(f, "sampler2DMSArray<{}>", scalar_ty),
+            (CubeMapArray, Sampled) => write!(f, "samplerCubeArray<{}>", scalar_ty),
+            (Image2DRect, Sampled) => write!(f, "sampler2DRect<{}>", scalar_ty),
+            (ImageBuffer, Sampled) => write!(f, "samplerBuffer<{}>", scalar_ty),
 
-            (Image1D, Depth) => f.write_str("sampler1DShadow"),
-            (Image2D, Depth) => f.write_str("sampler2DShadow"),
-            (CubeMap, Depth) => f.write_str("samplerCubeShadow"),
-            (Image1DArray, Depth) => f.write_str("sampler1DArrayShadow"),
-            (Image2DArray, Depth) => f.write_str("sampler2DArrayShadow"),
-            (CubeMapArray, Depth) => f.write_str("samplerCubeShadowArray"),
-            (Image2DRect, Depth) => write!(f, "sampler2DRectShadow"),
+            (Image1D, Depth) => write!(f, "sampler1DShadow<{}>", scalar_ty),
+            (Image2D, Depth) => write!(f, "sampler2DShadow<{}>", scalar_ty),
+            (CubeMap, Depth) => write!(f, "samplerCubeShadow<{}>", scalar_ty),
+            (Image1DArray, Depth) => write!(f, "sampler1DArrayShadow<{}>", scalar_ty),
+            (Image2DArray, Depth) => write!(f, "sampler2DArrayShadow<{}>", scalar_ty),
+            (CubeMapArray, Depth) => write!(f, "samplerCubeShadowArray<{}>", scalar_ty),
+            (Image2DRect, Depth) => write!(f, "sampler2DRectShadow<{}>", scalar_ty),
             _ => Err(fmt::Error::default()),
         }
     }
@@ -302,22 +328,34 @@ impl SubpassDataArrangement {
 
 #[derive(PartialEq, Eq, Hash, Clone)]
 pub struct SubpassDataType {
-    pub scalar_ty: ScalarType,
+    /// Scalar type of subpass data access result. In most cases it's `Some`,
+    /// but the SPIR-V specification allows it to be `OpTypeVoid`. I have never
+    /// encounter one tho.
+    pub scalar_ty: Option<ScalarType>,
+    /// Image arrangement which encodes multisampling state.
     pub arng: SubpassDataArrangement,
 }
 impl SubpassDataType {
-    pub fn new(scalar_ty: ScalarType, arng: SubpassDataArrangement) -> Self {
+    pub fn new(scalar_ty: Option<ScalarType>, arng: SubpassDataArrangement) -> Self {
         SubpassDataType { scalar_ty, arng }
     }
 }
 impl fmt::Debug for SubpassDataType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
+        let scalar_ty = {
+            let scalar_ty = self.scalar_ty.as_ref();
+            if let Some(scalar_ty) = scalar_ty {
+                format!("{:?}", scalar_ty)
+            } else {
+                "void".to_owned()
+            }
+        };
         match self.arng {
             SubpassDataArrangement::SubpassData => {
-                write!(f, "subpassData<{:?}>", self.scalar_ty)
+                write!(f, "subpassData<{}>", scalar_ty)
             },
             SubpassDataArrangement::SubpassDataMS => {
-                write!(f, "subpassDataMS<{:?}>", self.scalar_ty)
+                write!(f, "subpassDataMS<{}>", scalar_ty)
             },
         }
     }
@@ -395,12 +433,10 @@ pub struct StructMember {
     pub offset: usize,
     pub ty: Type,
 }
-#[derive(PartialEq, Eq, Default, Clone)]
+#[derive(PartialEq, Eq, Default, Clone, Hash)]
 pub struct StructType {
-    name: Option<String>,
-    members: Vec<StructMember>, // Offset and type.
-    // BTreeMap to keep the order for hashing.
-    name_map: BTreeMap<String, usize>,
+    pub name: Option<String>,
+    pub members: Vec<StructMember>, // Offset and type.
 }
 impl StructType {
     pub(crate) fn new(name: Option<String>) -> StructType {
@@ -418,60 +454,6 @@ impl StructType {
             .map(|last| last.offset + last.ty.nbyte().unwrap_or(0))
             .unwrap_or(0)
     }
-    pub fn nmember(&self) -> usize { self.members.len() }
-    pub fn get_member(&self, i: usize) -> Option<&'_ StructMember> {
-        self.members.get(i)
-    }
-    pub fn get_member_by_name(&self, name: &str) -> Option<&'_ StructMember> {
-        self.name_map.get(name).and_then(|x| self.get_member(*x))
-    }
-    pub fn get_member_name(&self, i: usize) -> Option<&'_ str> {
-        self.name_map.iter()
-            .find_map(|(name, &j)| if i == j { Some(name.as_ref()) } else { None })
-    }
-    /// Merge another structure type's member into this structure type.
-    pub(crate) fn merge(&mut self, src_struct_ty: &StructType) -> Result<()> {
-        use crate::hash;
-        let dst_struct_ty = self;
-        let member_offset = dst_struct_ty.members.len();
-        let member_appendix = src_struct_ty.members.iter().cloned();
-        dst_struct_ty.members.extend(member_appendix);
-        for (name, &member_idx) in src_struct_ty.name_map.iter() {
-            if let Some(&old_member_idx) = dst_struct_ty.name_map.get(name) {
-                let old_hash = hash(&dst_struct_ty.members[old_member_idx]);
-                let new_hash = hash(&src_struct_ty.members[member_idx]);
-                if old_hash != new_hash {
-                    return Err(Error::MismatchedManifest);
-                }
-            } else {
-                dst_struct_ty.name_map
-                    .insert(name.to_owned(), member_offset + member_idx);
-            }
-        }
-        Ok(())
-    }
-    /// Push a structure member to this type. Note that the method can fail when
-    /// the member's name has already been registered in the current type.
-    pub(crate) fn push_member(&mut self, member: StructMember) -> Result<()> {
-        let i = self.members.len();
-        if let Some(name) = member.name.as_ref() {
-            if self.name_map.insert(name.to_owned(), i).is_some() {
-                return Err(Error::NAME_COLLISION);
-            }
-        }
-        self.members.push(member);
-        Ok(())
-    }
-}
-impl Hash for StructType {
-    fn hash<H: Hasher>(&self, state: &mut H) {
-        self.members.hash(state);
-        // NOTE: This enforces that the names for a same member in each stage
-        // have to be the same to be correctly reflected.
-        for x in self.name_map.values() {
-            x.hash(state);
-        }
-    }
 }
 impl fmt::Debug for StructType {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
@@ -482,8 +464,7 @@ impl fmt::Debug for StructType {
         }
         for (i, member) in self.members.iter().enumerate() {
             if i != 0 { f.write_str(", ")?; }
-            if let Some(name) = self.name_map.iter()
-                .find_map(|(name, &idx)| if idx == i { Some(name) } else { None }) {
+            if let Some(name) = &member.name {
                 write!(f, "{}: {:?}", name, member.ty)?;
             } else {
                 write!(f, "{}: {:?}", i, member.ty)?;
@@ -511,6 +492,8 @@ macro_rules! declr_ty_accessor {
 #[derive(PartialEq, Eq, Hash, Clone)]
 // #[non_exhaustive] // TODO: (penguinliong) For SPIR-Q v0.5.
 pub enum Type {
+    /// Literally nothing. You shouldn't find this in reflection results.
+    Void(),
     /// A single value, which can be a signed or unsigned integer, a floating
     /// point number, or a boolean value.
     Scalar(ScalarType),
@@ -540,6 +523,7 @@ impl Type {
     pub fn nbyte(&self) -> Option<usize> {
         use Type::*;
         match self {
+            Void() => None,
             Scalar(scalar_ty) => Some(scalar_ty.nbyte()),
             Vector(vec_ty) => Some(vec_ty.nbyte()),
             Matrix(mat_ty) => Some(mat_ty.nbyte()),
@@ -552,43 +536,11 @@ impl Type {
             AccelStruct() => None,
         }
     }
-    pub fn resolve<S: AsRef<Sym>>(&self, sym: S) -> Option<MemberVariableResolution<'_>> {
-        let mut ty = self;
-        let mut offset = 0;
-        for seg in sym.as_ref().segs() {
-            // Ensure the outer-most type can be addressed.
-            if seg == Seg::Empty { break }
-            match ty {
-                Type::Struct(struct_ty) => {
-                    let member = match seg {
-                        Seg::Index(i) => struct_ty.get_member(i),
-                        Seg::Name(name) => struct_ty.get_member_by_name(name),
-                        _ => return None,
-                    }?;
-                    offset += member.offset;
-                    ty = &member.ty;
-                },
-                Type::Array(arr_ty) => {
-                    if let Seg::Index(idx) = seg {
-                        if let Some(nrepeat) = arr_ty.nrepeat() {
-                            if idx >= nrepeat as usize {
-                                return None;
-                            }
-                        }
-                        offset += arr_ty.stride() * idx;
-                        ty = &*arr_ty.proto_ty();
-                    } else { return None; }
-                },
-                _ => return None,
-            }
-        }
-        let member_var_res = MemberVariableResolution { offset, ty };
-        Some(member_var_res)
-    }
     // Iterate over all entries in the type tree.
     pub fn walk<'a>(&'a self) -> Walk<'a> { Walk::new(self) }
     declr_ty_accessor! {
         [Type]
+        is_void -> Void,
         is_scalar -> Scalar,
         is_vec -> Vector,
         is_mat -> Matrix,
@@ -604,6 +556,7 @@ impl Type {
 impl fmt::Debug for Type {
     fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
         match self {
+            Type::Void() => write!(f, "void"),
             Type::Scalar(scalar_ty) => scalar_ty.fmt(f),
             Type::Vector(vec_ty) => vec_ty.fmt(f),
             Type::Matrix(mat_ty) => mat_ty.fmt(f),
@@ -617,202 +570,3 @@ impl fmt::Debug for Type {
         }
     }
 }
-
-
-/// Structured representation of descriptor types.
-#[derive(PartialEq, Eq, Hash, Clone)]
-// #[non_exhaustive] // TODO: (penguinliong) For SPIR-Q v0.5.
-pub enum DescriptorType {
-    UniformBuffer(u32, Type),
-    StorageBuffer(u32, Type),
-    Image(u32, Type),
-    Sampler(u32),
-    SampledImage(u32, Type),
-    // Note that the third parameter is input attachment index, the first one
-    // is the binding count.
-    InputAttachment(u32, Type, u32),
-    AccelStruct(u32),
-}
-impl DescriptorType {
-    /// Get the size of buffer (in bytes) needed to contain all the data for
-    /// this buffer object or push constant buffer.
-    pub fn nbyte(&self) -> Option<usize> {
-        use DescriptorType::*;
-        match self {
-            UniformBuffer(_, ty) => ty.nbyte(),
-            StorageBuffer(_, ty) => ty.nbyte(),
-            _ => None,
-        }
-    }
-    /// Number of bindings at the binding point. All descriptors can have
-    /// multiple binding points. If the multi-binding is dynamic, 0 will be
-    /// returned.
-    ///
-    /// For more information about dynamic multi-binding, please refer to
-    /// Vulkan extension `VK_EXT_descriptor_indexing`, GLSL extension
-    /// `GL_EXT_nonuniform_qualifier` and SPIR-V extension
-    /// `SPV_EXT_descriptor_indexing`. Dynamic multi-binding is only supported
-    /// in Vulkan 1.2.
-    pub fn nbind(&self) -> u32 {
-        use DescriptorType::*;
-        match self {
-            UniformBuffer(nbind, _) => *nbind,
-            StorageBuffer(nbind, _) => *nbind,
-            Image(nbind, _) => *nbind,
-            Sampler(nbind) => *nbind,
-            SampledImage(nbind, _) => *nbind,
-            InputAttachment(nbind, _, _) => *nbind,
-            AccelStruct(nbind) => *nbind,
-        }
-    }
-    /// Resolve a symbol WITHIN the descriptor type. The symbol should not
-    /// be led by descriptor set numbers and binding point numbers.
-    pub fn resolve<S: AsRef<Sym>>(&self, sym: S) -> Option<MemberVariableResolution<'_>> {
-        use DescriptorType::*;
-        // Resolve for descriptor root.
-        match self {
-            UniformBuffer(_, ref ty) => ty,
-            StorageBuffer(_, ref ty) => ty,
-            _ => { return None },
-        }.resolve(sym)
-    }
-    // Iterate over all entries in the type tree.
-    pub fn walk<'a>(&'a self) -> Walk<'a> {
-        use DescriptorType::*;
-        let ty = match self {
-            UniformBuffer(_, ty) => ty,
-            StorageBuffer(_, ty) => ty,
-            Image(_, ty) => ty,
-            Sampler(_) => &Type::Sampler(),
-            SampledImage(_, ty) => ty,
-            InputAttachment(_, ty, _) => ty,
-            AccelStruct(_) => &Type::AccelStruct(),
-        };
-        Walk::new(ty)
-    }
-    declr_ty_accessor! {
-        [DescriptorType]
-        is_uniform_buf -> UniformBuffer,
-        is_storage_buf -> StorageBuffer,
-        is_img -> Image,
-        is_sampler -> Sampler,
-        is_sampled_img -> SampledImage,
-        is_input_attm -> InputAttachment,
-        is_accel_struct -> AccelStruct,
-    }
-}
-impl fmt::Debug for DescriptorType {
-    fn fmt(&self, f: &mut fmt::Formatter) -> fmt::Result {
-        use DescriptorType::*;
-        match self {
-            UniformBuffer(nbind, ty) => write!(f, "{}x {:?}", nbind, ty),
-            StorageBuffer(nbind, ty) => write!(f, "{}x {:?}", nbind, ty),
-            Image(nbind, ty) => write!(f, "{}x {:?}", nbind, ty),
-            Sampler(nbind) => write!(f, "{}x sampler", nbind),
-            SampledImage(nbind, ty) => write!(f, "{}x {:?}", nbind, ty),
-            InputAttachment(nbind, ty, input_attm_idx) => write!(f, "{}x {:?}[{}]", nbind, ty, input_attm_idx),
-            AccelStruct(nbind) => write!(f, "{}x accelerationStructure", nbind),
-        }
-    }
-}
-
-pub struct MemberVariableRouting<'a> {
-    pub sym: Symbol,
-    pub offset: usize,
-    pub ty: &'a Type,
-}
-
-struct WalkFrame<'a> {
-    sym_stem: Option<Symbol>,
-    base_offset: usize,
-    ty: &'a Type,
-    i: usize,
-}
-pub struct Walk<'a> {
-    inner: Vec<WalkFrame<'a>>,
-}
-impl<'a> Walk<'a> {
-    pub fn new(ty: &'a Type) -> Walk<'a> {
-        let frame = WalkFrame {
-            sym_stem: None,
-            base_offset: 0,
-            ty: ty,
-            i: 0,
-        };
-        Walk { inner: vec![frame] }
-    }
-}
-impl<'a> Iterator for Walk<'a> {
-    type Item = MemberVariableRouting<'a>;
-    fn next(&mut self) -> Option<MemberVariableRouting<'a>> {
-        fn get_child_ty_offset_seg<'a>(ty: &'a Type, i: usize) -> Option<(&'a Type, usize, Seg<'a>)> {
-            match ty {
-                Type::Struct(struct_ty) => {
-                    let member = struct_ty.members.get(i)?;
-                    let seg = if let Some(ref name) = member.name {
-                        Seg::Name(name)
-                    } else {
-                        Seg::Index(i)
-                    };
-                    Some((&member.ty, member.offset, seg))
-                },
-                Type::Array(arr_ty) => {
-                    // Unsized buffer are treated as 0-sized.
-                    if i < arr_ty.nrepeat.unwrap_or(0) as usize {
-                        Some((&arr_ty.proto_ty, arr_ty.stride() * i, Seg::Index(i)))
-                    } else { None }
-                },
-                _ => None,
-            }
-        }
-        enum LoopEnd<'a> {
-            Push(WalkFrame<'a>),
-            PopReturn(MemberVariableRouting<'a>),
-        }
-        loop {
-            // If used, this field will be filled with the next frame to be
-            // pushed at the back of the walk stack; or the last frame will be
-            // popped if the field is kept `None`.
-            let loop_end = if let Some(frame) = self.inner.last_mut() {
-                if let Some((child_ty, offset, seg)) = get_child_ty_offset_seg(frame.ty, frame.i) {
-                    frame.i += 1; // Step member.
-                    let sym = if let Some(sym_stem) = &frame.sym_stem {
-                        let mut sym = sym_stem.clone();
-                        sym.push(&seg);
-                        sym
-                    } else { seg.into() };
-                    let offset = frame.base_offset + offset;
-                    let ty = child_ty;
-                    if child_ty.is_struct() || child_ty.is_arr() {
-                        // Found composite type, step into it.
-                        LoopEnd::Push(WalkFrame { sym_stem: Some(sym), base_offset: offset, ty, i: 0 })
-                    } else {
-                        // Return directly if it's not a composite type.
-                        return Some(MemberVariableRouting { sym, offset, ty });
-                    }
-                } else {
-                    // Here can be reached only when the first frame's type is
-                    // neither an array nor a struct; or a later frame's
-                    // composite type's elements has been exhausted.
-                    let ty = frame.ty;
-                    let offset = frame.base_offset;
-                    let sym = frame.sym_stem.clone().unwrap_or_default();
-                    LoopEnd::PopReturn(MemberVariableRouting { sym, offset, ty })
-                }
-            } else {
-                // We have exhausted all types we have, including the root type
-                // of walk.
-                return None;
-            };
-            match loop_end {
-                LoopEnd::Push(frame) => {
-                    self.inner.push(frame)
-                },
-                LoopEnd::PopReturn(route) => {
-                    self.inner.pop();
-                    return Some(route);
-                }
-            }
-        }
-    }
-}
